# coding: utf-8

"""
    Mentortools Access Server

    Mentortools service for Access Packages with lazy access resolution

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import re  # noqa: F401
import io
import warnings

from pydantic import validate_arguments, ValidationError
from typing import overload, Optional, Union, Awaitable

from pydantic import StrictBool, StrictInt, StrictStr, conint

from typing import Optional

from access_server_connector.models.package_in import PackageIn
from access_server_connector.models.package_patch_in import PackagePatchIn
from access_server_connector.models.package_rule_in import PackageRuleIn
from access_server_connector.models.product_link_in import ProductLinkIn
from access_server_connector.models.wrapped_response_id_out import WrappedResponseIdOut
from access_server_connector.models.wrapped_response_int import WrappedResponseInt
from access_server_connector.models.wrapped_response_list_member_ownership_out import WrappedResponseListMemberOwnershipOut
from access_server_connector.models.wrapped_response_list_package_out import WrappedResponseListPackageOut
from access_server_connector.models.wrapped_response_list_product_link_out import WrappedResponseListProductLinkOut
from access_server_connector.models.wrapped_response_list_rule_out import WrappedResponseListRuleOut
from access_server_connector.models.wrapped_response_package_detail_out import WrappedResponsePackageDetailOut
from access_server_connector.models.wrapped_response_public_package_out import WrappedResponsePublicPackageOut

from access_server_connector.api_client import ApiClient
from access_server_connector.api_response import ApiResponse
from access_server_connector.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class PackagesApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    async def count_members_v1_packages_access_package_id_members_count_get(self, access_package_id : StrictInt, portal_id : StrictInt, active_only : Optional[StrictBool] = None, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id2 : Optional[StrictInt] = None, **kwargs) -> WrappedResponseInt:  # noqa: E501
        """Count members who have access to this package  # noqa: E501

        Query: portal_id, active_only?. Total matching members (for pagination).  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param portal_id: (required)
        :type portal_id: int
        :param active_only:
        :type active_only: bool
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseInt
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the count_members_v1_packages_access_package_id_members_count_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.count_members_v1_packages_access_package_id_members_count_get_with_http_info(access_package_id, portal_id, active_only, origin, referer, portal_id2, **kwargs)  # noqa: E501

    @validate_arguments
    async def count_members_v1_packages_access_package_id_members_count_get_with_http_info(self, access_package_id : StrictInt, portal_id : StrictInt, active_only : Optional[StrictBool] = None, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id2 : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Count members who have access to this package  # noqa: E501

        Query: portal_id, active_only?. Total matching members (for pagination).  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param portal_id: (required)
        :type portal_id: int
        :param active_only:
        :type active_only: bool
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseInt, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'access_package_id',
            'portal_id',
            'active_only',
            'origin',
            'referer',
            'portal_id2'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method count_members_v1_packages_access_package_id_members_count_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['access_package_id'] is not None:
            _path_params['access_package_id'] = _params['access_package_id']


        # process the query parameters
        _query_params = []
        if _params.get('portal_id') is not None:  # noqa: E501
            _query_params.append(('portal_id', _params['portal_id']))

        if _params.get('active_only') is not None:  # noqa: E501
            _query_params.append(('active_only', _params['active_only']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id2'] is not None:
            _header_params['portal_id'] = _params['portal_id2']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseInt",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages/{access_package_id}/members/count', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def count_packages_v1_packages_count_get(self, portal_id : StrictInt, is_active : Optional[StrictBool] = None, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id2 : Optional[StrictInt] = None, **kwargs) -> WrappedResponseInt:  # noqa: E501
        """Count a portal's access packages  # noqa: E501

        Query: portal_id, is_active?. Total matching packages (for pagination).  # noqa: E501

        :param portal_id: (required)
        :type portal_id: int
        :param is_active:
        :type is_active: bool
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseInt
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the count_packages_v1_packages_count_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.count_packages_v1_packages_count_get_with_http_info(portal_id, is_active, origin, referer, portal_id2, **kwargs)  # noqa: E501

    @validate_arguments
    async def count_packages_v1_packages_count_get_with_http_info(self, portal_id : StrictInt, is_active : Optional[StrictBool] = None, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id2 : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Count a portal's access packages  # noqa: E501

        Query: portal_id, is_active?. Total matching packages (for pagination).  # noqa: E501

        :param portal_id: (required)
        :type portal_id: int
        :param is_active:
        :type is_active: bool
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseInt, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'portal_id',
            'is_active',
            'origin',
            'referer',
            'portal_id2'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method count_packages_v1_packages_count_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('portal_id') is not None:  # noqa: E501
            _query_params.append(('portal_id', _params['portal_id']))

        if _params.get('is_active') is not None:  # noqa: E501
            _query_params.append(('is_active', _params['is_active']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id2'] is not None:
            _header_params['portal_id'] = _params['portal_id2']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseInt",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages/count', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def create_package_v1_packages_post(self, package_in : PackageIn, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> WrappedResponseIdOut:  # noqa: E501
        """Create an access package  # noqa: E501


        :param package_in: (required)
        :type package_in: PackageIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseIdOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the create_package_v1_packages_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.create_package_v1_packages_post_with_http_info(package_in, origin, referer, portal_id, **kwargs)  # noqa: E501

    @validate_arguments
    async def create_package_v1_packages_post_with_http_info(self, package_in : PackageIn, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Create an access package  # noqa: E501


        :param package_in: (required)
        :type package_in: PackageIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseIdOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'package_in',
            'origin',
            'referer',
            'portal_id'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_package_v1_packages_post" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id'] is not None:
            _header_params['portal_id'] = _params['portal_id']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['package_in'] is not None:
            _body_params = _params['package_in']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseIdOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def delete_package_v1_packages_access_package_id_delete(self, access_package_id : StrictInt, force : Optional[StrictBool] = None, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> WrappedResponseIdOut:  # noqa: E501
        """Delete an access package  # noqa: E501

        Free deletion only when nobody holds it; with grants returns 409 unless force=true (then all holders lose access, as if they had never bought it). Prefer PATCH is_active=false to retire.  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param force:
        :type force: bool
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseIdOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the delete_package_v1_packages_access_package_id_delete_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.delete_package_v1_packages_access_package_id_delete_with_http_info(access_package_id, force, origin, referer, portal_id, **kwargs)  # noqa: E501

    @validate_arguments
    async def delete_package_v1_packages_access_package_id_delete_with_http_info(self, access_package_id : StrictInt, force : Optional[StrictBool] = None, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Delete an access package  # noqa: E501

        Free deletion only when nobody holds it; with grants returns 409 unless force=true (then all holders lose access, as if they had never bought it). Prefer PATCH is_active=false to retire.  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param force:
        :type force: bool
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseIdOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'access_package_id',
            'force',
            'origin',
            'referer',
            'portal_id'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_package_v1_packages_access_package_id_delete" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['access_package_id'] is not None:
            _path_params['access_package_id'] = _params['access_package_id']


        # process the query parameters
        _query_params = []
        if _params.get('force') is not None:  # noqa: E501
            _query_params.append(('force', _params['force']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id'] is not None:
            _header_params['portal_id'] = _params['portal_id']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseIdOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages/{access_package_id}', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def delete_rule_v1_packages_access_package_id_rules_rule_id_delete(self, access_package_id : StrictInt, rule_id : StrictInt, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> WrappedResponseIdOut:  # noqa: E501
        """Remove an access rule from a package  # noqa: E501

        Revokes the resource for all holders on their next read.  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param rule_id: (required)
        :type rule_id: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseIdOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the delete_rule_v1_packages_access_package_id_rules_rule_id_delete_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.delete_rule_v1_packages_access_package_id_rules_rule_id_delete_with_http_info(access_package_id, rule_id, origin, referer, portal_id, **kwargs)  # noqa: E501

    @validate_arguments
    async def delete_rule_v1_packages_access_package_id_rules_rule_id_delete_with_http_info(self, access_package_id : StrictInt, rule_id : StrictInt, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Remove an access rule from a package  # noqa: E501

        Revokes the resource for all holders on their next read.  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param rule_id: (required)
        :type rule_id: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseIdOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'access_package_id',
            'rule_id',
            'origin',
            'referer',
            'portal_id'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_rule_v1_packages_access_package_id_rules_rule_id_delete" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['access_package_id'] is not None:
            _path_params['access_package_id'] = _params['access_package_id']

        if _params['rule_id'] is not None:
            _path_params['rule_id'] = _params['rule_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id'] is not None:
            _header_params['portal_id'] = _params['portal_id']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseIdOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages/{access_package_id}/rules/{rule_id}', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def get_package_by_slug_v1_packages_by_slug_slug_get(self, slug : StrictStr, **kwargs) -> WrappedResponsePublicPackageOut:  # noqa: E501
        """Get public package data by its direct-link slug (unauthenticated)  # noqa: E501


        :param slug: (required)
        :type slug: str
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponsePublicPackageOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the get_package_by_slug_v1_packages_by_slug_slug_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.get_package_by_slug_v1_packages_by_slug_slug_get_with_http_info(slug, **kwargs)  # noqa: E501

    @validate_arguments
    async def get_package_by_slug_v1_packages_by_slug_slug_get_with_http_info(self, slug : StrictStr, **kwargs) -> ApiResponse:  # noqa: E501
        """Get public package data by its direct-link slug (unauthenticated)  # noqa: E501


        :param slug: (required)
        :type slug: str
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponsePublicPackageOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'slug'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_package_by_slug_v1_packages_by_slug_slug_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['slug'] is not None:
            _path_params['slug'] = _params['slug']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponsePublicPackageOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages/by-slug/{slug}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def get_package_v1_packages_access_package_id_get(self, access_package_id : StrictInt, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> WrappedResponsePackageDetailOut:  # noqa: E501
        """Get a single access package (with its rules)  # noqa: E501


        :param access_package_id: (required)
        :type access_package_id: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponsePackageDetailOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the get_package_v1_packages_access_package_id_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.get_package_v1_packages_access_package_id_get_with_http_info(access_package_id, origin, referer, portal_id, **kwargs)  # noqa: E501

    @validate_arguments
    async def get_package_v1_packages_access_package_id_get_with_http_info(self, access_package_id : StrictInt, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Get a single access package (with its rules)  # noqa: E501


        :param access_package_id: (required)
        :type access_package_id: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponsePackageDetailOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'access_package_id',
            'origin',
            'referer',
            'portal_id'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_package_v1_packages_access_package_id_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['access_package_id'] is not None:
            _path_params['access_package_id'] = _params['access_package_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id'] is not None:
            _header_params['portal_id'] = _params['portal_id']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponsePackageDetailOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages/{access_package_id}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def get_rules_v1_packages_access_package_id_rules_get(self, access_package_id : StrictInt, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> WrappedResponseListRuleOut:  # noqa: E501
        """List a package's access rules  # noqa: E501

        The rules currently set on the package (for displaying the selected state).  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseListRuleOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the get_rules_v1_packages_access_package_id_rules_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.get_rules_v1_packages_access_package_id_rules_get_with_http_info(access_package_id, origin, referer, portal_id, **kwargs)  # noqa: E501

    @validate_arguments
    async def get_rules_v1_packages_access_package_id_rules_get_with_http_info(self, access_package_id : StrictInt, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """List a package's access rules  # noqa: E501

        The rules currently set on the package (for displaying the selected state).  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseListRuleOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'access_package_id',
            'origin',
            'referer',
            'portal_id'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_rules_v1_packages_access_package_id_rules_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['access_package_id'] is not None:
            _path_params['access_package_id'] = _params['access_package_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id'] is not None:
            _header_params['portal_id'] = _params['portal_id']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseListRuleOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages/{access_package_id}/rules', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def link_product_v1_packages_access_package_id_products_put(self, access_package_id : StrictInt, product_link_in : ProductLinkIn, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> WrappedResponseIdOut:  # noqa: E501
        """Link a marketplace product to a package  # noqa: E501


        :param access_package_id: (required)
        :type access_package_id: int
        :param product_link_in: (required)
        :type product_link_in: ProductLinkIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseIdOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the link_product_v1_packages_access_package_id_products_put_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.link_product_v1_packages_access_package_id_products_put_with_http_info(access_package_id, product_link_in, origin, referer, portal_id, **kwargs)  # noqa: E501

    @validate_arguments
    async def link_product_v1_packages_access_package_id_products_put_with_http_info(self, access_package_id : StrictInt, product_link_in : ProductLinkIn, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Link a marketplace product to a package  # noqa: E501


        :param access_package_id: (required)
        :type access_package_id: int
        :param product_link_in: (required)
        :type product_link_in: ProductLinkIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseIdOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'access_package_id',
            'product_link_in',
            'origin',
            'referer',
            'portal_id'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method link_product_v1_packages_access_package_id_products_put" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['access_package_id'] is not None:
            _path_params['access_package_id'] = _params['access_package_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id'] is not None:
            _header_params['portal_id'] = _params['portal_id']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['product_link_in'] is not None:
            _body_params = _params['product_link_in']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseIdOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages/{access_package_id}/products', 'PUT',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def list_members_v1_packages_access_package_id_members_get(self, access_package_id : StrictInt, portal_id : StrictInt, active_only : Optional[StrictBool] = None, limit : Optional[conint(strict=True, le=200, ge=1)] = None, offset : Optional[conint(strict=True, ge=0)] = None, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id2 : Optional[StrictInt] = None, **kwargs) -> WrappedResponseListMemberOwnershipOut:  # noqa: E501
        """List members who have access to this package (paginated)  # noqa: E501

        Query: portal_id, active_only?, limit, offset. Use /members/count for the total.  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param portal_id: (required)
        :type portal_id: int
        :param active_only:
        :type active_only: bool
        :param limit:
        :type limit: int
        :param offset:
        :type offset: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseListMemberOwnershipOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the list_members_v1_packages_access_package_id_members_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.list_members_v1_packages_access_package_id_members_get_with_http_info(access_package_id, portal_id, active_only, limit, offset, origin, referer, portal_id2, **kwargs)  # noqa: E501

    @validate_arguments
    async def list_members_v1_packages_access_package_id_members_get_with_http_info(self, access_package_id : StrictInt, portal_id : StrictInt, active_only : Optional[StrictBool] = None, limit : Optional[conint(strict=True, le=200, ge=1)] = None, offset : Optional[conint(strict=True, ge=0)] = None, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id2 : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """List members who have access to this package (paginated)  # noqa: E501

        Query: portal_id, active_only?, limit, offset. Use /members/count for the total.  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param portal_id: (required)
        :type portal_id: int
        :param active_only:
        :type active_only: bool
        :param limit:
        :type limit: int
        :param offset:
        :type offset: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseListMemberOwnershipOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'access_package_id',
            'portal_id',
            'active_only',
            'limit',
            'offset',
            'origin',
            'referer',
            'portal_id2'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_members_v1_packages_access_package_id_members_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['access_package_id'] is not None:
            _path_params['access_package_id'] = _params['access_package_id']


        # process the query parameters
        _query_params = []
        if _params.get('portal_id') is not None:  # noqa: E501
            _query_params.append(('portal_id', _params['portal_id']))

        if _params.get('active_only') is not None:  # noqa: E501
            _query_params.append(('active_only', _params['active_only']))

        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        if _params.get('offset') is not None:  # noqa: E501
            _query_params.append(('offset', _params['offset']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id2'] is not None:
            _header_params['portal_id'] = _params['portal_id2']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseListMemberOwnershipOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages/{access_package_id}/members', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def list_packages_v1_packages_get(self, portal_id : StrictInt, is_active : Optional[StrictBool] = None, limit : Optional[conint(strict=True, le=200, ge=1)] = None, offset : Optional[conint(strict=True, ge=0)] = None, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id2 : Optional[StrictInt] = None, **kwargs) -> WrappedResponseListPackageOut:  # noqa: E501
        """List a portal's access packages (paginated)  # noqa: E501

        Query: portal_id, is_active?, limit, offset. Use /count for the total.  # noqa: E501

        :param portal_id: (required)
        :type portal_id: int
        :param is_active:
        :type is_active: bool
        :param limit:
        :type limit: int
        :param offset:
        :type offset: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseListPackageOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the list_packages_v1_packages_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.list_packages_v1_packages_get_with_http_info(portal_id, is_active, limit, offset, origin, referer, portal_id2, **kwargs)  # noqa: E501

    @validate_arguments
    async def list_packages_v1_packages_get_with_http_info(self, portal_id : StrictInt, is_active : Optional[StrictBool] = None, limit : Optional[conint(strict=True, le=200, ge=1)] = None, offset : Optional[conint(strict=True, ge=0)] = None, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id2 : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """List a portal's access packages (paginated)  # noqa: E501

        Query: portal_id, is_active?, limit, offset. Use /count for the total.  # noqa: E501

        :param portal_id: (required)
        :type portal_id: int
        :param is_active:
        :type is_active: bool
        :param limit:
        :type limit: int
        :param offset:
        :type offset: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id2:
        :type portal_id2: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseListPackageOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'portal_id',
            'is_active',
            'limit',
            'offset',
            'origin',
            'referer',
            'portal_id2'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_packages_v1_packages_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('portal_id') is not None:  # noqa: E501
            _query_params.append(('portal_id', _params['portal_id']))

        if _params.get('is_active') is not None:  # noqa: E501
            _query_params.append(('is_active', _params['is_active']))

        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        if _params.get('offset') is not None:  # noqa: E501
            _query_params.append(('offset', _params['offset']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id2'] is not None:
            _header_params['portal_id'] = _params['portal_id2']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseListPackageOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def list_product_links_v1_packages_access_package_id_products_get(self, access_package_id : StrictInt, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> WrappedResponseListProductLinkOut:  # noqa: E501
        """List the marketplace products linked to a package  # noqa: E501


        :param access_package_id: (required)
        :type access_package_id: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseListProductLinkOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the list_product_links_v1_packages_access_package_id_products_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.list_product_links_v1_packages_access_package_id_products_get_with_http_info(access_package_id, origin, referer, portal_id, **kwargs)  # noqa: E501

    @validate_arguments
    async def list_product_links_v1_packages_access_package_id_products_get_with_http_info(self, access_package_id : StrictInt, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """List the marketplace products linked to a package  # noqa: E501


        :param access_package_id: (required)
        :type access_package_id: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseListProductLinkOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'access_package_id',
            'origin',
            'referer',
            'portal_id'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_product_links_v1_packages_access_package_id_products_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['access_package_id'] is not None:
            _path_params['access_package_id'] = _params['access_package_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id'] is not None:
            _header_params['portal_id'] = _params['portal_id']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseListProductLinkOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages/{access_package_id}/products', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def patch_package_v1_packages_access_package_id_patch(self, access_package_id : StrictInt, package_patch_in : PackagePatchIn, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> WrappedResponseIdOut:  # noqa: E501
        """Toggle is_active / is_locked  # noqa: E501

        Toggle is_active (gates new grants) / is_locked (hard lock, cuts existing access).  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param package_patch_in: (required)
        :type package_patch_in: PackagePatchIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseIdOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the patch_package_v1_packages_access_package_id_patch_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.patch_package_v1_packages_access_package_id_patch_with_http_info(access_package_id, package_patch_in, origin, referer, portal_id, **kwargs)  # noqa: E501

    @validate_arguments
    async def patch_package_v1_packages_access_package_id_patch_with_http_info(self, access_package_id : StrictInt, package_patch_in : PackagePatchIn, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Toggle is_active / is_locked  # noqa: E501

        Toggle is_active (gates new grants) / is_locked (hard lock, cuts existing access).  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param package_patch_in: (required)
        :type package_patch_in: PackagePatchIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseIdOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'access_package_id',
            'package_patch_in',
            'origin',
            'referer',
            'portal_id'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method patch_package_v1_packages_access_package_id_patch" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['access_package_id'] is not None:
            _path_params['access_package_id'] = _params['access_package_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id'] is not None:
            _header_params['portal_id'] = _params['portal_id']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['package_patch_in'] is not None:
            _body_params = _params['package_patch_in']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseIdOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages/{access_package_id}', 'PATCH',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def set_rule_v1_packages_access_package_id_rules_put(self, access_package_id : StrictInt, package_rule_in : PackageRuleIn, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> WrappedResponseIdOut:  # noqa: E501
        """Set an access rule on a package  # noqa: E501


        :param access_package_id: (required)
        :type access_package_id: int
        :param package_rule_in: (required)
        :type package_rule_in: PackageRuleIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseIdOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the set_rule_v1_packages_access_package_id_rules_put_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.set_rule_v1_packages_access_package_id_rules_put_with_http_info(access_package_id, package_rule_in, origin, referer, portal_id, **kwargs)  # noqa: E501

    @validate_arguments
    async def set_rule_v1_packages_access_package_id_rules_put_with_http_info(self, access_package_id : StrictInt, package_rule_in : PackageRuleIn, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Set an access rule on a package  # noqa: E501


        :param access_package_id: (required)
        :type access_package_id: int
        :param package_rule_in: (required)
        :type package_rule_in: PackageRuleIn
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseIdOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'access_package_id',
            'package_rule_in',
            'origin',
            'referer',
            'portal_id'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method set_rule_v1_packages_access_package_id_rules_put" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['access_package_id'] is not None:
            _path_params['access_package_id'] = _params['access_package_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id'] is not None:
            _header_params['portal_id'] = _params['portal_id']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['package_rule_in'] is not None:
            _body_params = _params['package_rule_in']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseIdOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages/{access_package_id}/rules', 'PUT',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def unlink_product_v1_packages_access_package_id_products_link_id_delete(self, access_package_id : StrictInt, link_id : StrictInt, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> WrappedResponseIdOut:  # noqa: E501
        """Unlink a marketplace product from a package  # noqa: E501

        New purchases of the product stop granting this package (existing grants stay).  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param link_id: (required)
        :type link_id: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseIdOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the unlink_product_v1_packages_access_package_id_products_link_id_delete_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.unlink_product_v1_packages_access_package_id_products_link_id_delete_with_http_info(access_package_id, link_id, origin, referer, portal_id, **kwargs)  # noqa: E501

    @validate_arguments
    async def unlink_product_v1_packages_access_package_id_products_link_id_delete_with_http_info(self, access_package_id : StrictInt, link_id : StrictInt, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Unlink a marketplace product from a package  # noqa: E501

        New purchases of the product stop granting this package (existing grants stay).  # noqa: E501

        :param access_package_id: (required)
        :type access_package_id: int
        :param link_id: (required)
        :type link_id: int
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseIdOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'access_package_id',
            'link_id',
            'origin',
            'referer',
            'portal_id'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method unlink_product_v1_packages_access_package_id_products_link_id_delete" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['access_package_id'] is not None:
            _path_params['access_package_id'] = _params['access_package_id']

        if _params['link_id'] is not None:
            _path_params['link_id'] = _params['link_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id'] is not None:
            _header_params['portal_id'] = _params['portal_id']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseIdOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/packages/{access_package_id}/products/{link_id}', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    Mentortools Access Server

    Mentortools service for Access Packages with lazy access resolution

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from access_server_connector.models.override_out import OverrideOut  # noqa: E501

class TestOverrideOut(unittest.TestCase):
    """OverrideOut unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> OverrideOut:
        """Test OverrideOut
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `OverrideOut`
        """
        model = OverrideOut()  # noqa: E501
        if include_optional:
            return OverrideOut(
                resource_type = 'course',
                resource_id = 56,
                is_active = True,
                rule = access_server_connector.models.rule_body_out.RuleBodyOut(
                    access_mode = null, 
                    unlock_type = null, 
                    unlock_days = 56, 
                    unlock_weekday = 56, 
                    unlock_weekdays = [
                        56
                        ], 
                    unlock_date = 56, 
                    expiry_type = null, 
                    expiry_days = 56, 
                    expiry_date = 56, 
                    binding = null, ),
                created_by = 56,
                created_at = 56
            )
        else:
            return OverrideOut(
                resource_type = 'course',
                resource_id = 56,
                is_active = True,
                rule = access_server_connector.models.rule_body_out.RuleBodyOut(
                    access_mode = null, 
                    unlock_type = null, 
                    unlock_days = 56, 
                    unlock_weekday = 56, 
                    unlock_weekdays = [
                        56
                        ], 
                    unlock_date = 56, 
                    expiry_type = null, 
                    expiry_days = 56, 
                    expiry_date = 56, 
                    binding = null, ),
                created_by = 56,
                created_at = 56,
        )
        """

    def testOverrideOut(self):
        """Test OverrideOut"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Mentortools Access Server

    Mentortools service for Access Packages with lazy access resolution

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class LockReason(str, Enum):
    """
    LockReason
    """

    """
    allowed enum values
    """
    LAUNCH_DATE = 'launch_date'
    EXPIRED = 'expired'
    LOCKED = 'locked'
    HIDDEN = 'hidden'

    @classmethod
    def from_json(cls, json_str: str) -> LockReason:
        """Create an instance of LockReason from a JSON string"""
        return LockReason(json.loads(json_str))





# coding: utf-8

"""
    Mentortools Access Server

    Mentortools service for Access Packages with lazy access resolution

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt
from access_server_connector.models.resource_type import ResourceType
from access_server_connector.models.rule_body import RuleBody

class OverrideItem(BaseModel):
    """
    OverrideItem
    """
    member_id: StrictInt = Field(...)
    resource_type: ResourceType = Field(...)
    resource_id: StrictInt = Field(...)
    is_active: Optional[StrictBool] = True
    rule: Optional[RuleBody] = None
    __properties = ["member_id", "resource_type", "resource_id", "is_active", "rule"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> OverrideItem:
        """Create an instance of OverrideItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of rule
        if self.rule:
            _dict['rule'] = self.rule.to_dict()
        # set to None if rule (nullable) is None
        # and __fields_set__ contains the field
        if self.rule is None and "rule" in self.__fields_set__:
            _dict['rule'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> OverrideItem:
        """Create an instance of OverrideItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return OverrideItem.parse_obj(obj)

        _obj = OverrideItem.parse_obj({
            "member_id": obj.get("member_id"),
            "resource_type": obj.get("resource_type"),
            "resource_id": obj.get("resource_id"),
            "is_active": obj.get("is_active") if obj.get("is_active") is not None else True,
            "rule": RuleBody.from_dict(obj.get("rule")) if obj.get("rule") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Mentortools Access Server

    Mentortools service for Access Packages with lazy access resolution

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr

class PackageOut(BaseModel):
    """
    PackageOut
    """
    id: StrictInt = Field(...)
    portal_id: StrictInt = Field(...)
    title: StrictStr = Field(...)
    payment_type: StrictStr = Field(...)
    is_active: StrictBool = Field(...)
    is_locked: StrictBool = Field(...)
    direct_link_slug: Optional[StrictStr] = Field(...)
    sales_page_url: Optional[StrictStr] = Field(...)
    expired_message: Optional[StrictStr] = Field(...)
    created_at: StrictInt = Field(...)
    updated_at: StrictInt = Field(...)
    __properties = ["id", "portal_id", "title", "payment_type", "is_active", "is_locked", "direct_link_slug", "sales_page_url", "expired_message", "created_at", "updated_at"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PackageOut:
        """Create an instance of PackageOut from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if direct_link_slug (nullable) is None
        # and __fields_set__ contains the field
        if self.direct_link_slug is None and "direct_link_slug" in self.__fields_set__:
            _dict['direct_link_slug'] = None

        # set to None if sales_page_url (nullable) is None
        # and __fields_set__ contains the field
        if self.sales_page_url is None and "sales_page_url" in self.__fields_set__:
            _dict['sales_page_url'] = None

        # set to None if expired_message (nullable) is None
        # and __fields_set__ contains the field
        if self.expired_message is None and "expired_message" in self.__fields_set__:
            _dict['expired_message'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PackageOut:
        """Create an instance of PackageOut from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PackageOut.parse_obj(obj)

        _obj = PackageOut.parse_obj({
            "id": obj.get("id"),
            "portal_id": obj.get("portal_id"),
            "title": obj.get("title"),
            "payment_type": obj.get("payment_type"),
            "is_active": obj.get("is_active"),
            "is_locked": obj.get("is_locked"),
            "direct_link_slug": obj.get("direct_link_slug"),
            "sales_page_url": obj.get("sales_page_url"),
            "expired_message": obj.get("expired_message"),
            "created_at": obj.get("created_at"),
            "updated_at": obj.get("updated_at")
        })
        return _obj





# coding: utf-8

"""
    Mentortools Access Server

    Mentortools service for Access Packages with lazy access resolution

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from access_server_connector.api.individual_api import IndividualApi  # noqa: E501


class TestIndividualApi(unittest.IsolatedAsyncioTestCase):
    """IndividualApi unit test stubs"""

    async def asyncSetUp(self) -> None:
        self.api = IndividualApi()

    async def asyncTearDown(self) -> None:
        await self.api.api_client.close()

    async def test_batch_overrides_v1_individual_overrides_post(self) -> None:
        """Test case for batch_overrides_v1_individual_overrides_post

        Batch-write per-member access overrides  # noqa: E501
        """
        pass

    async def test_member_overrides_v1_individual_members_member_id_overrides_get(self) -> None:
        """Test case for member_overrides_v1_individual_members_member_id_overrides_get

        List a member's effective overrides (latest-wins)  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Mentortools Access Server

    Mentortools service for Access Packages with lazy access resolution

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from access_server_connector.models.rule_body_out import RuleBodyOut  # noqa: E501

class TestRuleBodyOut(unittest.TestCase):
    """RuleBodyOut unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> RuleBodyOut:
        """Test RuleBodyOut
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `RuleBodyOut`
        """
        model = RuleBodyOut()  # noqa: E501
        if include_optional:
            return RuleBodyOut(
                access_mode = 'available',
                unlock_type = 'immediate',
                unlock_days = 56,
                unlock_weekday = 56,
                unlock_weekdays = [
                    56
                    ],
                unlock_date = 56,
                expiry_type = 'none',
                expiry_days = 56,
                expiry_date = 56,
                binding = 'lifetime'
            )
        else:
            return RuleBodyOut(
                access_mode = 'available',
                unlock_type = 'immediate',
                unlock_days = 56,
                unlock_weekday = 56,
                unlock_weekdays = [
                    56
                    ],
                unlock_date = 56,
                expiry_type = 'none',
                expiry_days = 56,
                expiry_date = 56,
                binding = 'lifetime',
        )
        """

    def testRuleBodyOut(self):
        """Test RuleBodyOut"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Mentortools Access Server

    Mentortools service for Access Packages with lazy access resolution

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import re  # noqa: F401
import io
import warnings

from pydantic import validate_arguments, ValidationError
from typing import overload, Optional, Union, Awaitable

from pydantic import StrictInt, StrictStr

from typing import Optional

from access_server_connector.models.access_order_event import AccessOrderEvent
from access_server_connector.models.wrapped_response_consume_out import WrappedResponseConsumeOut
from access_server_connector.models.wrapped_response_health_out import WrappedResponseHealthOut

from access_server_connector.api_client import ApiClient
from access_server_connector.api_response import ApiResponse
from access_server_connector.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class AdminApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    async def consume_v1_admin_consume_post(self, access_order_event : AccessOrderEvent, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> WrappedResponseConsumeOut:  # noqa: E501
        """Apply an access.order-events payload  # noqa: E501

        Apply an access.order-events payload (simulates the Kafka consumer; ops/testing).  # noqa: E501

        :param access_order_event: (required)
        :type access_order_event: AccessOrderEvent
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseConsumeOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the consume_v1_admin_consume_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.consume_v1_admin_consume_post_with_http_info(access_order_event, origin, referer, portal_id, **kwargs)  # noqa: E501

    @validate_arguments
    async def consume_v1_admin_consume_post_with_http_info(self, access_order_event : AccessOrderEvent, origin : Optional[StrictStr] = None, referer : Optional[StrictStr] = None, portal_id : Optional[StrictInt] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Apply an access.order-events payload  # noqa: E501

        Apply an access.order-events payload (simulates the Kafka consumer; ops/testing).  # noqa: E501

        :param access_order_event: (required)
        :type access_order_event: AccessOrderEvent
        :param origin:
        :type origin: str
        :param referer:
        :type referer: str
        :param portal_id:
        :type portal_id: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseConsumeOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'access_order_event',
            'origin',
            'referer',
            'portal_id'
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method consume_v1_admin_consume_post" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['origin'] is not None:
            _header_params['origin'] = _params['origin']

        if _params['referer'] is not None:
            _header_params['referer'] = _params['referer']

        if _params['portal_id'] is not None:
            _header_params['portal_id'] = _params['portal_id']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['access_order_event'] is not None:
            _body_params = _params['access_order_event']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['Bearer Authorization', 'ServerName Authorization']  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseConsumeOut",
            '422': "HTTPValidationError",
        }

        return await self.api_client.call_api(
            '/v1/admin/consume', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    async def health_health_get(self, **kwargs) -> WrappedResponseHealthOut:  # noqa: E501
        """Health check  # noqa: E501


        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: WrappedResponseHealthOut
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the health_health_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.health_health_get_with_http_info(**kwargs)  # noqa: E501

    @validate_arguments
    async def health_health_get_with_http_info(self, **kwargs) -> ApiResponse:  # noqa: E501
        """Health check  # noqa: E501


        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(WrappedResponseHealthOut, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
        ]
        _all_params.extend(
            [
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method health_health_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "WrappedResponseHealthOut",
        }

        return await self.api_client.call_api(
            '/health', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    Mentortools Access Server

    Mentortools service for Access Packages with lazy access resolution

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr, conlist
from access_server_connector.models.state import State

class CourseSummaryOut(BaseModel):
    """
    CourseSummaryOut
    """
    course_id: StrictInt = Field(...)
    state: State = Field(...)
    allowed: StrictBool = Field(...)
    reasons: conlist(StrictInt) = Field(...)
    expired_message: Optional[StrictStr] = None
    __properties = ["course_id", "state", "allowed", "reasons", "expired_message"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CourseSummaryOut:
        """Create an instance of CourseSummaryOut from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if expired_message (nullable) is None
        # and __fields_set__ contains the field
        if self.expired_message is None and "expired_message" in self.__fields_set__:
            _dict['expired_message'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CourseSummaryOut:
        """Create an instance of CourseSummaryOut from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CourseSummaryOut.parse_obj(obj)

        _obj = CourseSummaryOut.parse_obj({
            "course_id": obj.get("course_id"),
            "state": obj.get("state"),
            "allowed": obj.get("allowed"),
            "reasons": obj.get("reasons"),
            "expired_message": obj.get("expired_message")
        })
        return _obj





# coding: utf-8

"""
    Mentortools Access Server

    Mentortools service for Access Packages with lazy access resolution

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from access_server_connector.models.access_mode import AccessMode  # noqa: E501

class TestAccessMode(unittest.TestCase):
    """AccessMode unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testAccessMode(self):
        """Test AccessMode"""
        # inst = AccessMode()

if __name__ == '__main__':
    unittest.main()
